const express = require("express");
const morgan = require("morgan");
const cors = require("cors");
const routes = require("./routes/index.routes");
const bodyParser = require("body-parser");
const path = require("path");
const https = require("https");
const http = require("http");
const fs = require("fs");
const multer = require('multer');
const socketIo = require('socket.io');
const axios = require('axios');
const app = express();
require("dotenv").config({ path: __dirname + "/./../.env" });
const dir = __dirname + "/../../../../ssl/certs/";
const gzappy = require("gzappy-js");
const {  } = require("./util/cron")
const PedidosRepository = require('./infra/repository/pedidos.repository');
const AdminRepository = require('./infra/repository/admin.repository');
const PedidosController = require('./controller/pedidos.controller');
const AdminController = require("./controller/admin.controller");
const UsuariosRepository = require("./infra/repository/usuarios.repository")
const UsuariosController = require("./controller/usuarios.controller");
const VendedoresController = require("./controller/vendedores.controller")
const VendedoresRepository = require("./infra/repository/vendedores.repository");
const FuncionarioController = require("./controller/funcionario.controller");

// Configuração CORS
const corsOptions = {
    origin: "*", // Ajuste conforme necessário
    methods: ["GET", "POST", "PUT", "PATCH", "DELETE"],
    allowedHeaders: ["Content-Type", "Authorization"],
    credentials: true
};

app.use(cors(corsOptions));
app.use(express.json());
app.use(morgan("dev"));
app.use(bodyParser.json());
app.use(routes);

const server = process.env.NODE_ENV === "production"
    ? https.createServer(
        {
            key: fs.readFileSync(path.join("/home/aliancapedidos/ssl/keys/d7634_bb22f_2d5d975fdd58f7fb7faebc6aea18538f.key")),
            cert: fs.readFileSync(path.join("/home/aliancapedidos/ssl/certs/api_aliancapedidos_com_br_d7634_bb22f_1751805841_3feda855c756b3dd5a7e7845b4d659b9.crt")),
        },
        app
      )
    : http.createServer(app);


// Configuração do Socket.IO
const io = socketIo(server, {
    cors: {
        origin: "*",
        methods: ["GET", "POST", "PUT", "PATCH", "DELETE"],
        allowedHeaders: ["Content-Type"],
        credentials: true
    },
});

const pedidosRepository = new PedidosRepository(db);
const pedidosController = new PedidosController(pedidosRepository);

const adminRepository = new AdminRepository(db)
const adminController = new AdminController(adminRepository)

const usuariosRepository = new UsuariosRepository(db)
const usuariosController = new UsuariosController(usuariosRepository)

const vendedoresRepository = new VendedoresRepository(db)
const vendedoresController = new VendedoresController(vendedoresRepository)

const funcionarioRepository = new FuncionarioController(db)
const funcionarioController = new FuncionarioController(funcionarioRepository)

const { agendarMensagensPromocionaisParaTodos } = require("./util/cron");
 

io.on('connection', (socket) => {
    pedidosController.setupSocketListeners(socket);

   

    socket.on('listarHistorico', (data_inicio, data_fim) => {

        adminController.listUserPedidosConluidoSocket(data_inicio, data_fim)
            .then(resultado => {

                
                // Emite o resultado
                socket.emit("listarHistoricoController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listarHistoricoController", { error: error.message });
            });
    });
    socket.on('listarHistorico', (data_inicio, data_fim) => {
        setInterval(() => {

        adminController.listUserPedidosConluidoSocket(data_inicio, data_fim)
            .then(resultado => {
 
                
               
                socket.emit("listarHistoricoController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listarHistoricoController", { error: error.message });
            });
        }, 30000);
    });
         

  
    socket.on('listarGrafico', (dataInicio, dataFim) => {
        pedidosController.listFormSocket(dataInicio, dataFim)
            .then(resultado => {
                
                
                // Exibe as datas recebidas
                socket.emit("listarGraficoController", resultado);
            })
            .catch(error => {
                console.error("Erro: ", error);
                socket.emit("listarGraficoController", { error: error.message });
            });
    });
    
    socket.on('listarGrafico', (dataInicio, dataFim) => {
    setInterval(() => {
            pedidosController.listFormSocket(dataInicio, dataFim)
                .then(resultado => {
                    
                    
                    // Exibe as datas recebidas
                    socket.emit("listarGraficoController", resultado);
                })
                .catch(error => {
                    console.error("Erro: ", error);
                    socket.emit("listarGraficoController", { error: error.message });
                });
            }, 30000);
        });


    adminController.list()
        .then(dados => {
            socket.emit("respostaController", dados);
        })
        .catch(error => {
            console.error(error);
            socket.emit("respostaController", error);
        });
    setInterval(() => {

        adminController.list()
            .then(dados => {
                socket.emit("respostaController", dados);
            })
            .catch(error => {
                console.error(error);
                socket.emit("respostaController", error);
            });
    }, 10000);


    socket.on('listarUserData', (id_user) => {

        usuariosController.listUserDataSocket(id_user)
            .then(resultado => {


                socket.emit("listarUserDataController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listarUserDataController", { error: error.message });
            });
    });
    socket.on('listarUserData', (id_user) => {
        setInterval(() => {

        usuariosController.listUserDataSocket(id_user)
            .then(resultado => {
 
                
               
                socket.emit("listarUserDataController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listarUserDataController", { error: error.message });
            });
        }, 10000);
    });

    /// VENDEDORES


    socket.on('listUltimosPedidos', (id_vendedor) => {

        vendedoresController.listUltimosPedidos(id_vendedor)
            .then(resultado => {


                socket.emit("listUltimosPedidosController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listUltimosPedidosController", { error: error.message });
            });
    });
    socket.on('listUltimosPedidos', (id_vendedor) => {
        setInterval(() => {

            vendedoresController.listUltimosPedidos(id_vendedor)
            .then(resultado => {
 
                socket.emit("listUltimosPedidosController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listUltimosPedidosController", { error: error.message });
            });
        }, 10000);
    });



    socket.on('listCountAllVendedor', (id_vendedor) => {
        vendedoresController.listCountAllVendedor(id_vendedor)
            .then(resultado => {


                socket.emit("listCountAllVendedorController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listCountAllVendedorController", { error: error.message });
            });
    });
    socket.on('listCountAllVendedor', (id_vendedor) => {
        setInterval(() => {

            vendedoresController.listCountAllVendedor(id_vendedor)
            .then(resultado => {
 
                socket.emit("listCountAllVendedorController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listCountAllVendedorController", { error: error.message });
            });
        }, 10000);
    });


    socket.on('listMetasMensal', (id_vendedor) => {

        vendedoresController.listMetasMensal(id_vendedor)
            .then(resultado => {


                socket.emit("listMetasMensalController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listMetasMensalController", { error: error.message });
            });
    });
    socket.on('listMetasMensal', (id_vendedor) => {
        setInterval(() => {

            vendedoresController.listMetasMensal(id_vendedor)
            .then(resultado => {
 
                socket.emit("listMetasMensalController", resultado);
            })
            .catch(error => {
                console.error("Erro:", error);
                socket.emit("listMetasMensalController", { error: error.message });
            });
        }, 10000);
    });

    // ============ FUNCIONARIO ==================

    socket.on('listQtdeTotalFunc', (id_func) => {
        funcionarioController.listQtdeTotalProdutosFunc(id_func)
            .then(resultado => {
                
                
                // Exibe as datas recebidas
                socket.emit("listQtdeTotalFuncController", resultado);
            })
            .catch(error => {
                console.error("Erro: ", error);
                socket.emit("listQtdeTotalFuncController", { error: error.message });
            });
    });
    
    socket.on('listQtdeTotalFunc', (id_func) => {
    setInterval(() => {
            funcionarioController.listQtdeTotalProdutosFunc(id_func)
                .then(resultado => {
                    socket.emit("listQtdeTotalFuncController", resultado);
                })
                .catch(error => {
                    console.error("Erro: ", error);
                    socket.emit("listQtdeTotalFuncController", { error: error.message });
                });
            }, 30000);
        });


        socket.on('listDataFunc', (id_func) => {
            funcionarioController.listFuncDataSocket(id_func)
                .then(resultado => {
                    
                    
                    // Exibe as datas recebidas
                    socket.emit("listDataFuncController", resultado);
                })
                .catch(error => {
                    console.error("Erro: ", error);
                    socket.emit("listDataFuncController", { error: error.message });
                });
        });
        
        socket.on('listDataFunc', (id_func) => {
        setInterval(() => {
            funcionarioController.listFuncDataSocket(id_func)
                .then(resultado => {
                    socket.emit("listDataFuncController", resultado);
                })
                .catch(error => {
                    console.error("Erro: ", error);
                    socket.emit("listDataFuncController", { error: error.message });
                });
            }, 10000);
        });



         


    socket.on('disconnect', () => {});
});

const port = 3334;

if (process.env.NODE_ENV === "production") {
    server.listen(port, async () => {
      console.log(`Servidor rodando em https://api.aliancapedidos.com.br:${port}`);
      agendarMensagensPromocionaisParaTodos(); // agora ela estará definida
    });
  } else {
    server.listen(port, () => {
      console.log(`Servidor rodando em modo desenvolvimento: http://localhost:${port}`);
    });
}