const {
    decrypt,
    encrypt,
    generateHash,
    checkHash,
  } = require("../../config/auth/cryptography");
  const { genereteToken } = require("../../config/auth/jwt");
  const { encryptDonor, decryptDonor } = require("../../util/cryptDonor");
  const db = require("../database/db");
  const nodemailer = require("nodemailer");
  const crypto = require("crypto");
  const { response } = require("express");
  const { getMaxListeners } = require("../database/db");
  const path = require("path");
const sharp = require("sharp");
const { S3Client, PutObjectCommand, GetObjectCommand } = require('@aws-sdk/client-s3');
const { extraiTexto } = require("../../util/extraiTexto");
  require("dotenv").config({ path: __dirname + "/./../../../.env" });
  
  class produtosRepository {
  
    constructor() {
      this.db = db;
      this.s3Client = new S3Client({
        region: process.env.S3_REGION,
        credentials: {
          accessKeyId: process.env.AWS_ACCESS_KEY,
          secretAccessKey: process.env.AWS_SECRET_KEY,
        },
      });
    }
  

    /**
     * Registra um produto
     * @param {*} data dados do produto
     * @returns mensagem referente ao resultado obtido
     */
    register(data, file) {
        return new Promise(async (resolve, reject) => {

          let imageUrl = "";

          // Só faz upload se tiver imagem
          if (file) {
            const buffer = await sharp(file.buffer)
              .resize({ fit: "contain" })
              .toBuffer();
    
            const command = new PutObjectCommand({
              Bucket: process.env.S3_BUCKET,
              Key: file.originalname,
              Body: buffer,
              ContentType: file.mimetype,
            });
    
            await this.s3Client.send(command);
    
            imageUrl = `https://aliancapedidos.s3.us-east-2.amazonaws.com/${file.originalname}`;
          }
          
            console.log(data)

            this.db.query(
              `INSERT INTO produtos (id, nome, descricao, image, linha, categoria, ordem, preco1, preco2, preco3, preco4, preco5)
               VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`,
              [
                data.id,
                data.nome,
                data.descricao,
                imageUrl,
                data.linha,
                data.categoria,
                data.ordem,
                data.preco1,
                data.preco2,
                data.preco3,
                data.preco4,
                data.preco5,
              ],
              async (error, response) => {
                if (error) {
                  console.error(error);
                  return resolve({ error: "Erro ao cadastrar produto", code: 500 });
                }
                return resolve({ success: "produto adicionado com sucesso!" });
              }
            );
            
          });
    }
      

  
    /**
     * Lista os dados de uma unidade
     * @returns lista com todos os dados de uma unidade
     */
    listProdutosSimples(id, id_user) {
      return new Promise(async (resolve, reject) => {
        try {
          const usuarioLinha = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT linha FROM user WHERE id = ?`, 
              [2], 
              (error, usuario) => {  
                if (error) {
                  console.error("Erro ao buscar linha do usuário:", error);
                  return reject(error);
                }
                const linha = usuario.length && usuario[0].linha ? usuario[0].linha : null;
                resolve(linha);
              }
            );
          });
          
          let linhasArray = [];
          
          // Se o usuário não tiver linha, definir como ["brasa"]
          if (!usuarioLinha || usuarioLinha.trim() === "" ||  usuarioLinha.toLowerCase() === "null") {
            linhasArray = ["Brasa"];
          } else {
            linhasArray = usuarioLinha.split(",").map(linha => linha.trim());
          }
          
          // Montar a cláusula WHERE dinamicamente
          let query = "";
          if (linhasArray.length > 0) {
            const categoriasQuery = linhasArray.map(linha => `produtos.linha = '${linha}'`).join(" OR ");
            query = `(${categoriasQuery}) AND`;
          }
          
          console.log("Query gerada:", query);
          
          // Buscar produtos com base na linha
          const produtos = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT * FROM produtos WHERE ${query} produtos.exclusivo = false AND produtos.excluido != 1 ORDER BY produtos.ordem`,
              [],
              (error, produtos) => {
                if (error) {
                  console.error("Erro ao buscar produtos:", error);
                  return reject(error);
                }
                resolve(produtos);
              }
            );
          });
          
          // Log para depuração
          console.log("Linhas do usuário:", linhasArray);
          console.log("Produtos encontrados:", produtos);
          
          
          
          return resolve(produtos);
          
        } catch (error) {
          console.error(error);
          
          return reject(new Error(error));
        }
      });
    }

    listProdutosCompletos(id, id_user) {
      return new Promise(async (resolve, reject) => {
        try {
          // Busca a linha e o role do usuário
          const { linha, role } = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT linha, role FROM user WHERE id = ?`,
              [id_user],
              (error, usuario) => {
                if (error) {
                  console.error("Erro ao buscar usuário:", error);
                  return reject(error);
                }
                if (!usuario.length) return reject(new Error("Usuário não encontrado"));
                resolve(usuario[0]);
              }
            );
          });
    
          let linhasArray = [];
    
          // Se for admin, não filtrar por linha
          let query = `produtos.visivel = true AND produtos.excluido != true`;
    
          if (role !== "ADMIN") {
            // Se o usuário não tiver linha, definir como ["Brasa"]
            if (!linha || linha.trim() === "" || linha.toLowerCase() === "null") {
              linhasArray = ["Brasa"];
            } else {
              linhasArray = linha.split(",").map(l => l.trim());
            }
    
            if (linhasArray.length > 0) {
              const categoriasQuery = linhasArray
                .map(linha => `produtos.linha = '${linha}'`)
                .join(" OR ");
              query += ` AND (${categoriasQuery})`;
            }
          }
    
          console.log("Query gerada:", query);
    
          // Buscar todos os produtos baseado na query
          const todos_produtos = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT * FROM produtos 
               WHERE ${query}
               ORDER BY produtos.ordem`,
              [],
              (error, resultado) => {
                if (error) return reject(new Error(error));
                resolve(resultado);
              }
            );
          });
    
          // Se for admin, construir linhasArray dinamicamente a partir dos produtos
          if (role === "ADMIN") {
            linhasArray = [...new Set(todos_produtos.map(p => p.linha))];
          }
    
          const produtosAgrupados = todos_produtos.reduce((acc, produto) => {
            if (!linhasArray.includes(produto.linha)) return acc;
    
            if (!acc[produto.linha]) acc[produto.linha] = {};
            if (!acc[produto.linha][produto.categoria]) acc[produto.linha][produto.categoria] = [];
    
            acc[produto.linha][produto.categoria].push({
              id: produto.id,
              nome: produto.nome,
              descricao: produto.descricao,
              image: produto.image,
              visivel: produto.visivel,
              exclusivo: produto.exclusivo,
              especifico: produto.especifico,
            });
    
            return acc;
          }, {});
    
          // Ordenação
          const ordenarCategoriasBrasa = (a, b, categorias) => {
            const prioridade = ["Hambúrguer", "Linha Premium"];
            const indexA = prioridade.indexOf(a);
            const indexB = prioridade.indexOf(b);
    
            if (indexA !== -1 && indexB !== -1) return indexA - indexB;
            if (indexA !== -1) return -1;
            if (indexB !== -1) return 1;
    
            return categorias[b].length - categorias[a].length;
          };
    
          const ordenarCategoriasGenerico = (a, b, categorias) => {
            return categorias[b].length - categorias[a].length;
          };
    
          const ordenarLinhas = (a, b) => {
            if (a === "Brasa") return -1;
            if (b === "Brasa") return 1;
    
            const totalA = Object.values(produtosAgrupados[a] || {}).reduce((acc, arr) => acc + arr.length, 0);
            const totalB = Object.values(produtosAgrupados[b] || {}).reduce((acc, arr) => acc + arr.length, 0);
            return totalB - totalA;
          };
    
          const produtosOrdenados = {};
          Object.keys(produtosAgrupados)
            .sort(ordenarLinhas)
            .forEach(linha => {
              const categorias = produtosAgrupados[linha];
              produtosOrdenados[linha] = {};
    
              const categoriasOrdenadas = Object.keys(categorias).sort((a, b) => {
                if (linha === "Brasa") {
                  return ordenarCategoriasBrasa(a, b, categorias);
                } else {
                  return ordenarCategoriasGenerico(a, b, categorias);
                }
              });
    
              categoriasOrdenadas.forEach(categoria => {
                produtosOrdenados[linha][categoria] = categorias[categoria];
              });
            });
    
          return resolve(produtosOrdenados);
        } catch (error) {
          console.error(error);
          return reject(new Error(error));
        }
      });
    }
    
    

      /**
       * Lista todos os produtos visiveis
       * @returns lista com todos os dados de um produto visivel
       */
         listProdutosVisiveis(contrato) {
          return new Promise(async (resolve, reject) => {
    
            try {

              if (contrato !== "RH") {


              this.db.query(`SELECT * FROM produtos WHERE produtos.visivel = true ORDER BY produtos.ordem `, [],
                async (error, produtos) => { 
                  if (error) { 
                    console.log(error);
                    return reject(new Error(error));
                  }
                  return resolve(produtos);
              });
            }
            else {

            this.db.query(`SELECT * FROM produtos_func ORDER BY produtos_func.ordem `, [],
              async (error, produtos) => { 
                if (error) { 
                  console.log(error);
                  return reject(new Error(error));
                }
                return resolve(produtos);
            });
          }
    
            } catch (error) {
              console.error(error);
              return reject(new Error(error));
            }
          });
        }

      /**
     * Lista os dados de uma unidade
     * @returns lista com todos os dados de uma unidade
     */
      listProdutosDeslogado() {
        return new Promise(async (resolve, reject) => {
          try {
            // Buscar todos os produtos visíveis e não excluídos
            const todos_produtos = await new Promise((resolve, reject) => {
              this.db.query(
                `SELECT * FROM produtos 
                 WHERE produtos.visivel = true 
                 AND produtos.excluido != true 
                 ORDER BY produtos.ordem`,
                [],
                (error, resultado) => {
                  if (error) return reject(new Error(error));
                  resolve(resultado);
                }
              );
            });
      
            // Agrupar por linha e categoria
            const produtosAgrupados = todos_produtos.reduce((acc, produto) => {
              if (!acc[produto.linha]) acc[produto.linha] = {};
              if (!acc[produto.linha][produto.categoria]) acc[produto.linha][produto.categoria] = [];
      
              acc[produto.linha][produto.categoria].push({
                id: produto.id,
                nome: produto.nome,
                descricao: produto.descricao,
                image: produto.image,
                visivel: produto.visivel,
                exclusivo: produto.exclusivo,
                especifico: produto.especifico,
              });
      
              return acc;
            }, {});
      
            // Funções de ordenação
            const ordenarCategoriasBrasa = (a, b, categorias) => {
              const prioridade = ["Hambúrguer", "Linha Premium"];
              const indexA = prioridade.indexOf(a);
              const indexB = prioridade.indexOf(b);
      
              if (indexA !== -1 && indexB !== -1) return indexA - indexB;
              if (indexA !== -1) return -1;
              if (indexB !== -1) return 1;
      
              return categorias[b].length - categorias[a].length;
            };
      
            const ordenarCategoriasGenerico = (a, b, categorias) => {
              return categorias[b].length - categorias[a].length;
            };
      
            const ordenarLinhas = (a, b) => {
              if (a === "Brasa") return -1;
              if (b === "Brasa") return 1;
      
              const totalA = Object.values(produtosAgrupados[a] || {}).reduce((acc, arr) => acc + arr.length, 0);
              const totalB = Object.values(produtosAgrupados[b] || {}).reduce((acc, arr) => acc + arr.length, 0);
              return totalB - totalA;
            };
      
            // Ordenar estrutura final
            const produtosOrdenados = {};
            Object.keys(produtosAgrupados)
              .sort(ordenarLinhas)
              .forEach(linha => {
                const categorias = produtosAgrupados[linha];
                produtosOrdenados[linha] = {};
      
                const categoriasOrdenadas = Object.keys(categorias).sort((a, b) => {
                  if (linha === "Brasa") {
                    return ordenarCategoriasBrasa(a, b, categorias);
                  } else {
                    return ordenarCategoriasGenerico(a, b, categorias);
                  }
                });
      
                categoriasOrdenadas.forEach(categoria => {
                  produtosOrdenados[linha][categoria] = categorias[categoria];
                });
              });
      
            return resolve(produtosOrdenados);
          } catch (error) {
            console.error(error);
            return reject(new Error(error));
          }
        });
      }
      

    /**
     * Lista todos os produtos invisiveis
     * @returns lista todos os produtos invisiveis
     */
        listProdutosInvisiveis(id) {
        return new Promise(async (resolve, reject) => {
  
          try {
            this.db.query(`SELECT * FROM produtos WHERE produtos.exclusivo = false AND produtos.excluido != 1 ORDER BY produtos.ordem `, [],
              async (error, produtos) => { 
                if (error) { 
                  console.log(error);
                  return reject(new Error(error));
                }
                return resolve(produtos);
            });
  
  
          } catch (error) {
            console.error(error);
            return reject(new Error(error));
          }
        });
      }

    listTodosProdutosAdmin(id) {
      return new Promise(async (resolve, reject) => {
        var query = id
          ? `WHERE id = ?`
          :``
  
        try {

          this.db.query(
            `SELECT * FROM produtos WHERE produtos.excluido != 1 ORDER BY produtos.ordem ${query}`, 
            (error, produtos) => {  
              if (error) { 
                console.log(error);
                return reject(new Error(error));
              }
          
              // Corrigindo a iteração sobre os produtos
              // for (const produto of produtos) {
              //   if (!produto.image.startsWith("https://brasapedidos.com.br/gallery/var/albums/brasaPedidos/")) {
              //     produto.image = `https://brasapedidos.s3.us-east-2.amazonaws.com/${produto.image}`;
              //   }
              // }
          
              return resolve(produtos);
            }
          );
          
        } catch (error) {
          console.error(error);
          
          return reject(new Error(error));
        }
      });
    }

        /**
     * Lista os todos os preco1 dos produtos
     * @returns lista com todos os dados de uma unidade
     */
        listProdutosPreco1(id) {
          return new Promise(async (resolve, reject) => {
            var query = id
              ? `WHERE id = ?`
              :``
      
            try {
              this.db.query(`SELECT produtos.id AS id_produto, produtos.nome, produtos.descricao,produtos.image, produtos.tipo, produtos.categoria, produtos.preco1
                            FROM  produtos
                            WHERE
                            categoria = 2`, [id ? id : ""], 
              async (error, produtos) => {
                
                if (error) { 
                  
                  console.log(error);
                  return reject(new Error(error));
                }
      
                return resolve(produtos);
              });
            } catch (error) {
              return reject(new Error(error));
            }
          });
        }

    /**
     * Lista todos os preco2 dos produtos
     * @returns lista com todos os dados de uma unidade
    */
      listProdutosPreco2(id) {
        return new Promise(async (resolve, reject) => {
          var query = id
            ? `WHERE id = ?`
            :``
    
          try {
            this.db.query(`SELECT produtos.id AS id_produto, produtos.nome, produtos.descricao,produtos.image, produtos.tipo, produtos.categoria, produtos.preco2
                          FROM  produtos
                          WHERE
                          categoria = 2`, [id ? id : ""], 
            async (error, produtos) => {
              
              if (error) { 
                
                console.log(error);
                return reject(new Error(error));
              }
    
              return resolve(produtos);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }

    /**
     * Lista todos os preco3 dos produtos
     * @returns lista com todos os dados de uma unidade
    */
    listProdutosPreco3(id) {
      return new Promise(async (resolve, reject) => {
        var query = id
          ? `WHERE id = ?`
          :``
  
        try {
          this.db.query(`SELECT produtos.id AS id_produto, produtos.nome, produtos.descricao,produtos.image, produtos.tipo, produtos.categoria, produtos.preco3
                        FROM  produtos
                        WHERE
                        categoria = 2`, [id ? id : ""], 
          async (error, produtos) => {
            
            if (error) { 
              
              console.log(error);
              return reject(new Error(error));
            }
  
            return resolve(produtos);
          });
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }


      
    /**
     * Lista todos produtos e seus descontos
     * @returns lista com todos os dados de produtos com descontos
     */
    listProdutosDescontos(id) {
      return new Promise(async (resolve, reject) => {
        var query = id
          ? `WHERE id = ?`
          :``
  
        try {
          this.db.query(`SELECT produtos.id AS id_produto, produtos.nome, produtos.descricao, produtos.image, produtos.linha, produtos.categoria,
                        descontos.desconto
                        FROM descontos, produtos
                        WHERE
                        descontos.id_produto = produtos.id `, [id ? id : ""], 
          async (error, produtos) => {
            
            if (error) { 
              
              console.log(error);
              return reject(new Error(error));
            }
  
            return resolve(produtos);
          });
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }

    /**
     * Lista todos produtos e seus descontos de um usuario
     * @returns lista com todos os dados de produtos com descontos
     */
      listProdutosDescontosByUser(id_user) {
        return new Promise(async (resolve, reject) => {
    
          try {
            this.db.query(`SELECT 
                        produtos.id AS id_produto, 
                        produtos.nome, 
                        COALESCE(descontos.desconto, 0) AS desconto,
                        COALESCE(descontos.operacao, '') AS operacao
                    FROM produtos
                    LEFT JOIN descontos ON descontos.id_produto = produtos.id 
                        AND descontos.id_user = ?
                    WHERE produtos.excluido != 1 AND produtos.excluido IS NOT NULL
                    ORDER BY linha
            `, [id_user ? id_user : ""], 
            async (error, produtos) => {
              
              if (error) { 
                console.log(error);
                return reject(new Error(error));
              }
              
              this.db.query(
                `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
                [id_user, new Date(), "Solicitação de descontos"],
                (error, results) => {
                  if (error) {
                    console.error("Error inserting logs: ", error);
                    return reject(error);
                  }
                  resolve();
                }
              );
    
              return resolve(produtos);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }

    /**
     * Lista todos produtos por categoria 
     * @returns lista com todos os dados de produtos com descontos
     */
        listProdutosCategoria(num) {
          return new Promise(async (resolve, reject) => {

      
            try {
              this.db.query(`SELECT produtos.id AS id_produto, produtos.nome, produtos.descricao, produtos.categoria FROM produtos
                          WHERE
                          produtos.categoria = ? `, [num ? num : ""], 
              async (error, produtos) => {
                
                if (error) { 
                  
                  console.log(error);
                  return reject(new Error(error));
                }
      
                return resolve(produtos);
              });
            } catch (error) {
              return reject(new Error(error));
            }
          });
        }

      /**
     * Lista os produtos de Top AS
     * @returns lista com todos os dados de produtos Top As
     */
    listProdutosTopAs(id_user) {
      return new Promise(async (resolve, reject) => {
        var query = id_user ? `u.id = ? AND` : ``


        try {
          this.db.query(`SELECT p.id, p.nome, p.descricao, p.image, p.tipo, p.categoria, p.preco1
              FROM produtos p
              WHERE 
                  EXISTS (
                      SELECT 
                          1
                      FROM 
                          user u
                      WHERE 
                        ${query}
                        u.tipo like '%Top As%')`, 
          [id_user ? id_user : ""], 
          async (error, produtos) => {
            
            if (error) { 
              
              console.log(error);
              return reject(new Error(error));
            }

            return resolve(produtos);
          });
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }

      /**
     * Lista os produtos de categoria 1 por tipo de usuario calcula o desconto por item
     * @returns lista com todos os produtos por tipo de usuario com desconto aplicado
     */
      listProdutosByUserTipo(id_user, tipo) {
        return new Promise((resolve, reject) => {
          try {
            this.db.query(
              `SELECT p.id, p.nome, p.descricao, p.image, p.linha, p.categoria, p.ordem,
                CASE
                  WHEN ? = "distribuidor" THEN p.preco1
                  WHEN ? = "atacado" THEN p.preco2
                  WHEN ? = "top as"  THEN p.preco3
                ELSE NULL
                END AS preco
                FROM produtos p
                WHERE p.categoria = 1
                ORDER BY p.ordem`,
              [`${tipo}`, `${tipo}`, `${tipo}`],
              (error, produtos) => {
                if (error) {
                  console.log(error);
                  return reject(new Error(error));
                }

                return resolve(produtos);
              }
            );
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }

      /**
     * Lista os produtos de categoria 2 por tipo de usuario e calcule o desconto por item
     * @returns lista com todos os produtos por tipo de usuario com desconto aplicado
     */
      listProdutosByUserTipo2(tipo) {
        return new Promise((resolve, reject) => {
          try {
            this.db.query(
              `SELECT p.id, p.nome, p.descricao, p.image, p.linha, p.categoria, p.ordem,
                CASE
                  WHEN ? = "distribuidor" THEN p.preco1
                  WHEN ? = "atacado" THEN p.preco2
                  WHEN ? = "top as"  THEN p.preco3
                  WHEN ? = "sem tabela" THEN p.preco4
                  WHEN ? = "funcionario" THEN p.preco5
                ELSE NULL
                END AS preco
                FROM produtos p
                WHERE p.categoria = 2
                ORDER BY p.ordem`,
              [`${tipo}`, `${tipo}`, `${tipo}`],
              (error, produtos) => {
                if (error) {
                  console.log(error);
                  return reject(new Error(error));
                }
                return resolve(produtos);
              }
            );
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }

      listTabelaDePrecoUser(id_user) {
        return new Promise(async(resolve, reject) => {
          try {

            const user = await new Promise((resolve, reject) => {
              this.db.query(
                  `SELECT * from user WHERE id = ?`,[id_user],
                  (error, user) => {
                      if (error) {
                          console.error("Error: ", error);
                          return reject(error);
                      }
                      resolve(user[0]);
                  }
              );
          });

          let categoriaFilter = '';

          if (user.role === 'FUNC') categoriaFilter = 'p.categoria IN (1,2)';
          else if (user.role === 'USER') categoriaFilter = 'p.categoria = 1';

          const produtos = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT p.id, p.image, p.linha, p.categoria, p.ordem,
                CASE
                  WHEN ? = "distribuidor" THEN p.preco1
                  WHEN ? = "atacado" THEN p.preco2
                  WHEN ? = "top as"  THEN p.preco3
                  WHEN ? = "sem tabela" THEN p.preco4
                  WHEN ? = "funcionario" THEN p.preco5
                ELSE NULL
                END AS preco,
                CASE
                  WHEN ? = "distribuidor" THEN p.nome
                  WHEN ? = "atacado" THEN p.nome
                  WHEN ? = "top as"  THEN p.nome
                  WHEN ? = "sem tabela" THEN p.nome
                  WHEN ? = "funcionario" THEN p.nome_unidade
                ELSE NULL
                END AS nome,
                CASE
                  WHEN ? = "distribuidor" THEN p.descricao
                  WHEN ? = "atacado" THEN p.descricao
                  WHEN ? = "top as"  THEN p.descricao
                  WHEN ? = "sem tabela" THEN p.descricao
                  WHEN ? = "funcionario" THEN p.descricao_unidade
                ELSE NULL
                END AS descricao
                FROM produtos p
                WHERE ${categoriaFilter}
                ORDER BY p.ordem`,
              [`${user.tipo}`, `${user.tipo}`, `${user.tipo}`, `${user.tipo}`, `${user.tipo}`,
                `${user.tipo}`, `${user.tipo}`, `${user.tipo}`, `${user.tipo}`, `${user.tipo}`,
                `${user.tipo}`, `${user.tipo}`, `${user.tipo}`, `${user.tipo}`, `${user.tipo}`
              ],
              (error, produtos) => {
                if (error) {
                  console.log(error);
                  return reject(new Error(error));
                }
                return resolve(produtos);
              }
            );
          })

          
            return resolve(produtos)


          } catch (error) {
            return reject(new Error(error));
          }
        });
      }


      listProdutosByUserId(tipo, id_user, categoria) {
        return new Promise((resolve, reject) => {
          try {
            this.db.query(
              `SELECT p.id, p.nome, p.descricao, p.image, p.linha, p.categoria, p.ordem,
              d.desconto, d.operacao,
              CASE
                WHEN ? = 'distribuidor' THEN COALESCE(p.preco1, 0) 
                  WHEN ? = 'atacado' THEN COALESCE(p.preco2, 0)
                  WHEN ? = 'top as' THEN COALESCE(p.preco3, 0)
                ELSE null
              END AS preco
      FROM produtos p
      LEFT JOIN descontos d ON d.id_produto = p.id AND d.id_user = ?
      WHERE p.categoria = ? 
      ORDER BY p.ordem`,
      [`${tipo}`, `${tipo}`, `${tipo}`, id_user, categoria],
      (error, produtos) => {
        console.log(produtos);
        
        if (error) {
          console.log(error);
          return reject(new Error(error));
        }

        produtos.forEach((produto) => {
                  
      
          const desconto = produto.desconto ? parseFloat(produto.desconto) : 0;

          if (produto.operacao === "+") {
            produto.preco = produto.preco + desconto
          } 
          if (produto.operacao === "-") {
            produto.preco = produto.preco - desconto
          }
          if (produto.operacao === "%") {
            if (!isNaN(produto.preco) && !isNaN(desconto)) {
              produto.preco = produto.preco - (produto.preco * (desconto / 100));
            } 
            
          } 
          else {
            produto.preco = produto.preco; 
          }


          produto.preco = parseFloat(produto.preco);
        });

        return resolve(produtos);
      

      
      }
    );
  } catch (error) {
    return reject(new Error(error));
  }
});
}

      /**
     * Lista todos produtos de distribuidores
     * @returns lista com todos os produtos de distribuidores
     */
      listProdutosDistribuidor(id_user) {
        return new Promise(async (resolve, reject) => {
          var query = id_user ? `u.id = ? AND` : ``
  
  
          try {
            this.db.query(`SELECT p.id, p.nome, p.descricao, p.image, p.tipo, p.categoria, p.preco3
              FROM produtos p
              WHERE 
                  EXISTS (
                      SELECT 
                          1
                      FROM 
                          user u
                      WHERE 
                        ${query}
                        u.tipo like '%Distribuidor%')`, [id_user ? id_user : ""], 
            async (error, produtos) => {
              
              if (error) { 
                
                console.log(error);
                return reject(new Error(error));
              }
  
              return resolve(produtos);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }



      listLinhaProdutos() {
        return new Promise(async (resolve, reject) => {
  
  
          try {
            this.db.query(`SELECT distinct produtos.linha FROM produtos WHERE produtos.excluido != 1`, [], 
            async (error, produtos) => {
              
              if (error) { 
                console.log(error);
                return reject(new Error(error));
              }
  
              return resolve(produtos);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }
    
  
    /**
     * Deleta um produto 
     * @param {*} id Id do produto a ser excluído
     * @returns mensagem referente ao resultado obtido
     */
    delete(id) {
      return new Promise((resolve, reject) => {
  
        this.db.query('DELETE FROM produtos WHERE id = ?', [id],
        (error, response) => {
            if (error) {
                return resolve({ error: "Esta produtos possui vinculos!", code: 203 });
            }
            else {
                return resolve(true);
            }
        });
      });
    }

       /**
     * Deleta todos os pedidos de um usuario
     * @param {*} id_user Id do usuario a ser excluido
     * @returns mensagem referente ao resultado obtido
     */
       deletePedidoByUserId(id_user) {
        return new Promise((resolve, reject) => {
    
          this.db.query('DELETE FROM pedidos WHERE id_user = ?', [id_user],
          (error, response) => {
              if (error) {
                  return resolve({ error: "Este pedidos possui vinculos!", code: 203 });
              }
              else {
                  return resolve(true);
              }
          });
        });
      }
    
  
    /**
     * Atualiza o preco1 de um produto
     * @param {*} data dados atualizados do produto
     * @returns mensagem referente ao resultado obtido
     */
    updatePreco1(data) {
      return new Promise(async (resolve, reject) => {

    
          this.db.query(
            "UPDATE produtos SET preco1=? WHERE id = ?",
            [data.preco, data.id],
            (error, response) => {
              if (error) {
                return reject(new Error(error)); 
              }
    
               return resolve({ success: "Todos os preços dos produtos foram atualizados com sucesso!", code: 200 });
              
            }
          );
      });
    }

    /**
     * Atualiza o preco2 de um produto
     * @param {*} data dados atualizados do produto
     * @returns mensagem referente ao resultado obtido
     */
    updatePreco2(data) {
      return new Promise(async (resolve, reject) => {

    
        this.db.query(
          "UPDATE produtos SET preco2=? WHERE id = ?",
          [data.preco, data.id],
          (error, response) => {
            if (error) {
              return reject(new Error(error)); 
            }
  
             return resolve({ success: "Todos os preços dos produtos foram atualizados com sucesso!", code: 200 });
            
          }
        );
    });
  }

        /**
     * Atualiza preco3 de um produto
     * @param {*} data dados atualizados do produto
     * @returns mensagem referente ao resultado obtido
     */
    updatePreco3(data) {
      return new Promise(async (resolve, reject) => {

    
        this.db.query(
          "UPDATE produtos SET preco3=? WHERE id = ?",
          [data.preco, data.id],
          (error, response) => {
            if (error) {
              return reject(new Error(error)); 
            }
  
             return resolve({ success: "Todos os preços dos produtos foram atualizados com sucesso!", code: 200 });
            
          }
        );
    });
  }

    /**
     * Atualiza o produto 
     * @param {*} data dados atualizados do produto
     * @param {int} id id do produto a ser atualizado
     * @returns mensagem referente ao resultado obtido
     */
    updateProduto(data, file) {
      return new Promise(async (resolve, reject) => {
        try {
          console.log(data)
          let imageUrl = null;
    
          if (file) {
            const buffer = await sharp(file?.buffer)
              .resize({ fit: "contain" })
              .toBuffer();
    
            imageUrl = `https://aliancapedidos.s3.us-east-2.amazonaws.com/${file.originalname}`;
    
            const command = new PutObjectCommand({
              Bucket: process.env.S3_BUCKET,
              Key: file.originalname,
              Body: buffer,
              ContentType: file.mimetype
            });
    
            await this.s3Client.send(command);
          }
    
          const ordemValue = data.ordem ?? null;
    
          await new Promise((resolve, reject) => {
            this.db.query(
              `UPDATE produtos 
               SET 
                 nome = ?, 
                 descricao = ?, 
                 image = COALESCE(?, image),
                 ordem = COALESCE(?, ordem),
                 linha = ?, 
                 categoria = ?
               WHERE id = ?`,
              [
                data.nome,
                data.descricao,
                imageUrl,
                ordemValue,
                data.linha,
                data.categoria,
                data.id
              ],
              (error, response) => {
                if (error) {
                  console.error(error);
                  return reject(new Error(error));
                }
                resolve();
              }
            );
          });
    
          // Trata as tabelas
          if (typeof data.tabelas === "string" && data.tabelas.trim().startsWith("[")) {
            try {
              data.tabelas = JSON.parse(data.tabelas);
            } catch (error) {
              console.error("Erro ao converter 'tabela' para JSON:", error);
              data.tabelas = [];
            }
          } else {
            console.warn("'tabela' não é uma string JSON válida.");
            data.tabelas = [];
          }
    
          for (const tabela of data.tabelas) {
            await new Promise((resolve, reject) => {
              if (tabela.id) {
                this.db.query(
                  "UPDATE precos SET tabela=?, valor=? WHERE id = ?",
                  [tabela.tabela, tabela.preco, tabela.id],
                  (error, response) => {
                    if (error) {
                      console.error(error);
                      return reject(new Error(error));
                    }
                    resolve();
                  }
                );
              } else {
                this.db.query(
                  "INSERT INTO precos (id_produto, tabela, valor) VALUES (?, ?, ?)",
                  [data.id, tabela.tabela, tabela.preco],
                  (error, response) => {
                    if (error) {
                      console.error(error);
                      return reject(new Error(error));
                    }
                    resolve();
                  }
                );
              }
            });
          }
    
          return resolve({ success: "Produto atualizado com sucesso!", code: 200 });
        } catch (error) {
          console.error(error);
          return resolve({ error: "Erro ao atualizar produto", code: 500 });
        }
      });
    }
    
    

  setVisivelExclusivo(data) {
    return new Promise(async (resolve, reject) => {

  
      this.db.query(
        "UPDATE produtos SET visivel=?, exclusivo=? WHERE id = ?",
        [data.visivel, data.exclusivo, data.id],
        (error, response) => {
          if (error) {
            return reject(new Error(error)); 
          }

           return resolve({ success: "Todos os preços dos produtos foram atualizados com sucesso!", code: 200 });
          
        }
      );
  });
}

setExcluido(data) {
  return new Promise(async (resolve, reject) => {


    this.db.query(
      "UPDATE produtos SET excluido=? WHERE id = ?",
      [data.excluido, data.id],
      (error, response) => {
        if (error) {
          return reject(new Error(error)); 
        }

         return resolve({ success: "Produto excluido com sucesso!", code: 200 });
        
      }
    );
});
}


  testeFotos(id) {
    return new Promise(async (resolve, reject) => {

      try {

        this.db.query(
          `SELECT * FROM produtos WHERE id = ?`,[id],
          (error, produtos) => {  
            if (error) { 
              console.log(error);
              return reject(new Error(error));
            }
        
            const imageFormatada = extraiTexto(produtos[0].image);

            
        this.db.query(
          `UPDATE produtos SET image=? WHERE id = ?`,[imageFormatada, id],
          (error, produtos) => {  
            if (error) { 
              console.log(error);
              return reject(new Error(error));
            }
            return resolve({ success: "Produto atualizado com sucesso!", code: 200 });
          }
        );

            return resolve(produtos);
          }
        );
        
      } catch (error) {
        console.error(error);
        return reject(new Error(error));
      }
    });
  }




  }
  
module.exports = produtosRepository;
  